""" Class that visits the AST to update relative cell references.
    @author: Marc Provost
    @organization: McGill University
    @license: GNU General Public License
    @contact: marc.provost@mail.mcgill.ca
"""

from myast.ASTVisitor import ASTVisitor
from myast.Function import Function
from myast.RangeRef import RangeRef
from myast.CellRef import CellRef
from utility.ErrorUtility import *
from utility.TypeCheckUtility import TypeCheckUtility

errHandler  = ErrorUtility.getInstance()
typeChecker = TypeCheckUtility.getInstance()

class CopyPasteVisitor(ASTVisitor):
  """ Class that visits the AST to update relative cell references.
  """


  def __init__(self, sourceCol, sourceRow, destCol, destRow):
    if typeChecker.debug:
      typeChecker.typeCheck([sourceCol, sourceRow, destCol, destRow],
                      [int,int,int,int])
    self.__sourceCol=sourceCol
    self.__sourceRow=sourceRow
    self.__destCol=destCol
    self.__destRow=destRow

  def visitFunction(self, function):
    if typeChecker.debug: typeChecker.typeCheck([function], [Function])
    for arg in function.getArgs():
      arg.accept(self)

  def visitRangeRef(self, rangeRef):
    if typeChecker.debug: typeChecker.typeCheck([rangeRef], [RangeRef])
    cornerA=rangeRef.getFirstCorner()
    cornerB=rangeRef.getSecondCorner()
    cornerA.accept(self)
    cornerB.accept(self)
    rangeRef.setFirstCorner(cornerA)
    rangeRef.setSecondCorner(cornerB)

  def visitCellRef(self, cellRef):
    if typeChecker.debug: typeChecker.typeCheck([cellRef], [CellRef])
    if not cellRef.isAbsColumn():
      diff=cellRef.getColumn()-self.__sourceCol
      cellRef.setColumn(self.__destCol+diff)
    if not cellRef.isAbsRow():
      diff=cellRef.getRow()-self.__sourceRow
      cellRef.setRow(self.__destRow+diff)
