""" Module holding the functions used in formulas.
    @author: Marc Provost
    @organization: McGill University
    @license: GNU General Public License
    @contact: marc.provost@mail.mcgill.ca    
"""

from utility.TypeCheckUtility import TypeCheckUtility
from utility.ErrorUtility import ErrorUtility
from utility.ErrorUtility import LOG
from utility.ErrorUtility import WARNING
from utility.ErrorUtility import FATAL

typeChecker=TypeCheckUtility.getInstance()
errHandler=ErrorUtility.getInstance()

def sum(args):
  if typeChecker.debug:
    typeChecker.typeCheck([args],[list])
    for arg in args:
      typeChecker.typeCheck([arg], [(int,float)])
  if len(args)==0:
    v=ValueError("sum need at least 1 argument")
    errHandler.exceptionRaised(v, WARNING)
    return 0
    
  answer=args[0]
  for floatNum in args[1:]:
    answer+=floatNum
  return answer
  
def mul(args):
  if typeChecker.debug:
    typeChecker.typeCheck([args],[list])
    for arg in args:
      typeChecker.typeCheck([arg], [(int,float)])
  if len(args)==0:
    v=ValueError("sum need at least 1 argument")
    errHandler.exceptionRaised(v, WARNING)
    return 0
    
  answer=args[0]
  for floatNum in args[1:]:
    answer*=floatNum
  return answer

def exp(args):
  if typeChecker.debug:
    typeChecker.typeCheck([args],[list])
    for arg in args:
      typeChecker.typeCheck([arg], [(int,float)])
  if len(args)==0:
    v=ValueError("sum need at least 1 argument")
    errHandler.exceptionRaised(v, WARNING)
    return 0
    
  answer=args[0]
  for floatNum in args[1:]:    
    answer=pow(answer,floatNum)
  return answer

def invSum(args):
  if typeChecker.debug:
    typeChecker.typeCheck([args],[list])
    for arg in args:
      typeChecker.typeCheck([arg], [(int,float)])
      
  if len(args)!=1:
    v=ValueError("invSum need exactly 1 argument")
    errHandler.exceptionRaised(v, WARNING)
    return sum(args)*-1
  else:
    return args[0]*-1

def invMul(args):
  if typeChecker.debug:
    typeChecker.typeCheck([args],[list])
    for arg in args:
      typeChecker.typeCheck([arg], [(int,float)])
      
  if len(args)!=1:
    v=ValueError("invMul need exactly 1 argument")
    errHandler.exceptionRaised(v, WARNING)
    return mul(args)*-1
  else:
      return 1.0/args[0]

def unaryMinus(args):
  if typeChecker.debug:
    typeChecker.typeCheck([args],[list])
    for arg in args:
      typeChecker.typeCheck([arg], [(int,float)])
      
  if len(args)!=1:
    v=ValueError("invMul need exactly 1 argument")
    errHandler.exceptionRaised(v, WARNING)
    return mul(args)*-1
  else:
      return -1.0*args[0]  

def avg(args):
  if typeChecker.debug:
    typeChecker.typeCheck([args],[list])
    for arg in args:
      typeChecker.typeCheck([arg], [(int,float)])
      
  if len(args)==0:
    v=ValueError("avg need at least 1 argument")
    errHandler.exceptionRaised(v, WARNING)
    return 0 
  
  total=0
  for arg in args:
    total+=arg
  return total/len(args)

AVG=avg
Avg=avg
average=avg
Average=avg    
  
def med(args):
  if typeChecker.debug:
    typeChecker.typeCheck([args],[list])
    for arg in args:
      typeChecker.typeCheck([arg], [(int,float)])
      
  if len(args)==0:
    v=ValueError("med need at least 1 argument")
    errHandler.exceptionRaised(v, WARNING)
    return 0 
  
  args.sort()
  return args[int(len(args)/2)] 

MED=med
Med=med
median=med
Median=med    
  
def max(args):
  if typeChecker.debug:
    typeChecker.typeCheck([args],[list])
    for arg in args:
      typeChecker.typeCheck([arg], [(int,float)])
  
  if len(args)==0:
    v=ValueError("max need at least 1 argument")
    errHandler.exceptionRaised(v, WARNING)
    return 0 

  curMax=args[0]
  for arg in args[1:]:
    if curMax<arg:
      curMax=arg
  return curMax 

MAX=max
Max=max
maximum=max
Maximum=max    

def min(args):
  if typeChecker.debug:
    typeChecker.typeCheck([args],[list])
    for arg in args:
      typeChecker.typeCheck([arg], [(int,float)])
  
  if len(args)==0:
    v=ValueError("min need at least 1 argument")
    errHandler.exceptionRaised(v, WARNING)
    return 0 

  curMin=args[0]
  for arg in args[1:]:
    if curMin>arg:
      curMin=arg
  return curMin

MIN=min
Min=min
minimum=min
Minimum=min                  
              
                    
  
              
                
                  
